@@ -180,7 +180,11 @@ impl<I: Tokens> Parser<I> {
     }
 
     /// `tsParseEntityName`
-    fn parse_ts_entity_name(&mut self, allow_reserved_words: bool) -> PResult<TsEntityName> {
+    fn parse_ts_entity_name(
+        &mut self,
+        allow_reserved_words: bool,
+        allow_private_identifiers: bool,
+    ) -> PResult<TsEntityName> {
         debug_assert!(self.input.syntax().typescript());
 
         let init = self.parse_ident_name()?;
@@ -196,7 +200,7 @@ impl<I: Tokens> Parser<I> {
         let mut entity = TsEntityName::Ident(init);
         while eat!(self, '.') {
             let dot_start = cur_pos!(self);
-            if !is!(self, '#') && !is!(self, IdentName) {
+            if !allow_private_identifiers && !is!(self, '#') && !is!(self, IdentName) {
                 self.emit_err(
                     Span::new(dot_start, dot_start, Default::default()),
                     SyntaxError::TS1003,
@@ -205,10 +209,13 @@ impl<I: Tokens> Parser<I> {
             }
 
             let left = entity;
-            let right = if allow_reserved_words {
-                self.parse_ident_name()?
+            let right = if allow_private_identifiers {
+                self.parse_maybe_private_name()?
+                    .either(Into::into, Into::into)
+            } else if allow_reserved_words {
+                self.parse_ident_name()?.into()
             } else {
-                self.parse_ident(false, false)?
+                self.parse_ident(false, false)?.into()
             };
             entity = TsEntityName::TsQualifiedName(Box::new(TsQualifiedName { left, right }));
         }
@@ -225,7 +232,9 @@ impl<I: Tokens> Parser<I> {
 
         let has_modifier = self.eat_any_ts_modifier()?;
 
-        let type_name = self.parse_ts_entity_name(/* allow_reserved_words */ true)?;
+        let type_name = self.parse_ts_entity_name(
+            /* allow_reserved_words */ true, /* allow_private_identifiers */ false,
+        )?;
         trace_cur!(self, parse_ts_type_ref__type_args);
         let type_params = if !self.input.had_line_break_before_cur() && is!(self, '<') {
             Some(self.parse_ts_type_args()?)
@@ -317,7 +326,7 @@ impl<I: Tokens> Parser<I> {
         expect!(self, ')');
 
         let qualifier = if eat!(self, '.') {
-            self.parse_ts_entity_name(false).map(Some)?
+            self.parse_ts_entity_name(false, false).map(Some)?
         } else {
             None
         };
@@ -347,7 +356,7 @@ impl<I: Tokens> Parser<I> {
         } else {
             self.parse_ts_entity_name(
                 // allow_reserved_word
-                true,
+                true, true,
             )
             .map(From::from)?
         };
@@ -1126,8 +1135,10 @@ impl<I: Tokens> Parser<I> {
         if self.is_ts_external_module_ref()? {
             self.parse_ts_external_module_ref().map(From::from)
         } else {
-            self.parse_ts_entity_name(/* allow_reserved_words */ false)
-                .map(From::from)
+            self.parse_ts_entity_name(
+                /* allow_reserved_words */ false, /* allow_private_identifiers */ false,
+            )
+            .map(From::from)
         }
     }
 
